## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import re
import os



from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject
from Products.CMFPlone.utils import base_hasattr

import Products.Archetypes.Field

# from Products.LinguaFace import config


# security.declareProtected(Permission, methodNameAsString)
# security.declarePrivate(methodNameAsString)
# security.declarePublic(methodNameAsString)
# default manager permission is CMFCorePermissions.ManagePortal

class LinguaFaceTool(PropertyManager, UniqueObject, SimpleItem):
    """
        This tool holds the Google Analytics Id inserted on each page.
    """
    id = 'linguaface_tool'
    meta_type = 'LinguaFaceTool'
    plone_tool = 1
    title = "LinguaFace Tool"
    #Synchronise workflow between translations
    synchronise_translations_workflow = False

    _properties = (
        {'id':'synchronise_translations_workflow', 'type': 'boolean', 'mode':'w'},
    )

    overviewPath = os.path.join(os.path.dirname(__file__), 'zmi', 'overview')

    manage_options = (

        ({'label' : 'Overview',
          'action' : 'overview'
          },)

        + PropertyManager.manage_options
        + SimpleItem.manage_options
    )

    security = ClassSecurityInfo()
    overview = PageTemplateFile(overviewPath)
    security.declareProtected(CMFCorePermissions.ManagePortal, 'overview')

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container) :
        SimpleItem.manage_afterAdd(self, item, container)

    security.declarePrivate('getShortContentSubClass')
    def getShortContentSubClass(self, field, instance):
        """
            Override this function if you want to add your own Field handlers.
        """
        ### A False value indicate that the method returns nothing meaningfull
        return False

    security.declarePublic('getShortContent')
    def getShortContent(self, field, instance):
        """
            Return a short string value for each type of field
            field is the field instance that have to be "short-displayed"
            instance is the content instance
        """
        result = self.getShortContentSubClass(field, instance)
        if result:
            return result

        if isinstance(field, Products.Archetypes.Field.StringField) or\
           isinstance(field, Products.Archetypes.Field.IntegerField) or\
           isinstance(field, Products.Archetypes.Field.BooleanField):
            result = str(field.getAccessor(instance)()) ## return the complete string/integer/boolean
        elif isinstance(field, Products.Archetypes.Field.TextField):
            result = str(field.getAccessor(instance)())
            ##result = re.sub('<(?:(?:[^=]*="[^"]*")|[^>])*>', "", result)[:100]
            result = re.sub('<.*>', "", result)[:100]
        elif isinstance(field, (Products.Archetypes.Field.ImageField,
                                Products.Archetypes.Field.FileField,)):
            result = field.getFilename(instance)
        elif isinstance(field, Products.Archetypes.Field.ReferenceField):
            result = "<em>References</em>"
        else:
            result = "<em>No short content found (%s).</em>" % field.__class__.__name__

        if len(result) == 0:
            return "<em>Field is empty.</em>"
        else:
            return result

    security.declarePublic('isTranslatable')
    def isTranslatable(self, object):
        """ return true if object is translatable """
        return base_hasattr(object, 'getCanonicalPath')

InitializeClass(LinguaFaceTool)
